function simVal = computeHammingSimilarity(object, objectArray)
%computeHammingSimilarity - compute Hamming (counting) similarity.
% Computes the Hamming similarity between an object and an array of
% objects. The Hamming similarity is the number of features two
% objects have in common, and is also called the counting similarity.
%
% USAGE:
% simVal = computeHammingSimilarity(object, objectArray)
% object: a numeric row vector or a 1-by-M cell array of features.
% objectArray: a numeric matrix or a N-by-M cell array of objects. Each row
%     represents an object with M features. If object is numeric,
%     objectArray must also be numeric; if object is a cell array,
%     objectArray must also be a cell array.
% simVal: N-by-1 numeric vector of computed similarity values; each element
%     represents the similarity of object to each object in objectArray.
% 
% EXAMPLE 
% object = {'tall', 'wide', 'heavy, 1000};
% objectArray = {'short', 'wide', 'narrow', 1000; ...
%     'tall', 'wide', 'heavy', 20};
% simVal = computeHammingSimilarity(object, objectArray)
%
% simVal =
%
%     2
%     3

% Written by Luca Cazzanti, engineer.
% Applied Physics Laboratory, November 2006.
%$Id:$
%$DateTime:$
%$Author:$

%% Simple case: all object features are numeric, stored in a vector.
if isnumeric(object)
  simVal = local_computeNumericSimilarity(object, objectArray);
  return
end


%% Other case: heterogeneous features stored in a cell array.

% Compute the Hamming similarity separately for numeric and string
% features, then add the two to get the total similarity.

% Find indices of numeric and string features
[numericFeatureIdx, stringFeatureIdx] = local_separateFeatures(object);

% Compute numeric similarity
simValNumeric = 0;
objectNumericFeatures = object(1,numericFeatureIdx);
if ~isempty(objectNumericFeatures)
  objectNumericFeatures = cell2mat(objectNumericFeatures);
  objectArrayNumericFeatures = objectArray(:,numericFeatureIdx);
  objectArrayNumericFeatures = cell2mat(objectArrayNumericFeatures);
  simValNumeric = local_computeNumericSimilarity(objectNumericFeatures, ...
    objectArrayNumericFeatures);
end %if ~isempty

% Compute similarity for non-numeric features
objectStringFeatures = object(1,stringFeatureIdx);
objectArrayStringFeatures = objectArray(:,stringFeatureIdx);
simValString = local_computeStringSimilarity(objectStringFeatures, ...
  objectArrayStringFeatures);


% Final answer is ...
simVal = simValNumeric + simValString;
 
return


%% Local functions

% Find numeric and non-numeric features and return their column
% indeces.
function [numericFeatureIdx, stringFeatureIdx] = ...
  local_separateFeatures(object)
numericFeatureIdx = [];
stringFeatureIdx = [];
numFeatures = length(object);
for iFeature = 1:numFeatures
  if ischar( object{1,iFeature} ) == true
    stringFeatureIdx = [stringFeatureIdx iFeature];
  else
    numericFeatureIdx = [numericFeatureIdx iFeature];
  end %if ischar
end % for iFeature
return
% END local_saparateFeatures

% Compute the Hamming similarity of numeric features
function simVal = local_computeNumericSimilarity(object, objectArray)
[numObjects, numFeatures] = size(objectArray); %#ok
obj1Mat = repmat(object(:).', numObjects, 1);
simVal = sum( obj1Mat == objectArray , 2);
return
% END local_computeNumericSimilarity

% Compute the Hamming similarity for string features
function simVal = local_computeStringSimilarity(object, objectArray)
[numObjects, numFeatures] = size(objectArray); %#ok
simVal = zeros(numObjects,1);
for iObject = 1:numObjects
  simVal(iObject) = sum( strcmp(object, objectArray(iObject,:)) );
end
return
%END local_computeStringSimilarity
