function dataOut = generatePerturbedCentroidData(centroidMat, probVec, nData)
%generatePerturbedCentroidData - generate perturbed centroids-based data.
% Generates random binary vectors which are random perturbations of given
% binary vectors (centroids), according to user-provided probabilites.
% USAGE
% dataOut = generatePerturbedCentroidData(centroidMat, probVec, nData)
% centroidMat: N-by-M matrix; each row is a centroid and each column a
%     feature.
% probVec: N-by-1 vector of probabilities that a feature is flipped with respect
%     to its corresponding feature in the centroid. For each centroid, all
%     features have the same probability of being flipped.
% nData: number of data points to generate for each centroid. If a scalar
%     number, nData points are generated for each centroid; if a N-by-1 vector,
%     the ith centroid generates nData(i) points.
% dataOut: N-by-1 cell array. The ith element is a nData-by-M binary matrix
%     generated by randomly flipping the features of the ith centroid.

% Written by Luca Cazzanti, engineer.
% Applied Physics Laboratory, University of Washington, December 2006.
%$Id:$
%$DateTime:$
%$Author:$

[nCentroids, notUsed] = size(centroidMat); %#ok
if length(nData) == 1
  nData = nData * ones(nCentroids,1);
end

dataOut = cell(nCentroids,1);
for iCentroid = 1:nCentroids
  dataMat = repmat(centroidMat(iCentroid,:), nData(iCentroid), 1);
  randMat = rand(size(dataMat));
  featureToFlip = find(randMat < probVec(iCentroid));
  dataMat(featureToFlip) = bitcmp(dataMat(featureToFlip),1);
  dataOut{iCentroid} = dataMat;
end
  
return