function [trainMat, trainClass, testMat, testClass] = ...
  multiCentroid_generateTrainTestData(nTrain, nTest, nFeatures, ...
  nCentroidsPerClass, flipProbs)
%multiCentroid_generateTrainTestData() - generate multi-centroid data.
% Generates training and test data and associated labels
% (1, 2,...N), for multi-centroid classes of binary vectors.
% USAGE
% [trainMat, trainClass, testMat, testClass] = ...
%   multiCentroid_generateTrainTestData(nTrain, nTest, nFeatures, ...
%   nCentroidsPerClass, flipProbs);
%

% Written by Luca Cazzanti, engineer
% Applied Physics Laboratory, University of Washington, Dec 2006.

% Figure out number of classes and how many centroids per class
nClasses = length(flipProbs);
if length(nCentroidsPerClass) == 1 % same number of centroids per class
  nCentroidsPerClass = ones(nClasses,1) * nCentroidsPerClass;
elseif length(nCentroidsPerClass) ~= nClasses
  error(['nCentroidsPerClass must be a scalar or a vector of length ' ...
    'equal to the number of classes (the length of flipProbs)']);
end
nCentroids = sum(nCentroidsPerClass);

% The number of training and test samples associated with each centroid is
% random
histEdges = 0:(1/nCentroids):( (nCentroids + 1)/nCentroids ); % yes,+1
trainCentroidData = rand(nTrain,1);
nTrainPerCentroid = histc(trainCentroidData, histEdges);
nTrainPerCentroid(end-1:end) = [];%because of how histc works 
testCentroidData = rand(nTest,1);
nTestPerCentroid = histc(testCentroidData, histEdges);
nTestPerCentroid(end-1:end) = []; %because of how histc works
clear trainCentroidData testCentroidData

% Probabilites of flipping features for each centroid
probVec = [];
for iClass = 1:nClasses
  tmpVec = flipProbs(iClass) * ones(nCentroidsPerClass(iClass),1);
  probVec = [probVec; tmpVec];
end

% Matrix of centroids
centroidMat = round(rand(nCentroids, nFeatures));

% Generate training and test data.
trainData = generatePerturbedCentroidData(centroidMat, probVec, ...
  nTrainPerCentroid);
testData = generatePerturbedCentroidData(centroidMat, probVec, ...
  nTestPerCentroid);

% Put data in matrices and create vectors of class labels.
trainMat = [];
trainClass = [];
testMat = [];
testClass = [];
centroidCounter = 1;
for iClass = 1:nClasses
  for iCentroid = 1:nCentroidsPerClass(iClass)
    trainMat = [trainMat; trainData{centroidCounter}];
    trainCentroidClass = iClass * ones( size( trainData{centroidCounter}, 1), 1 );
    trainClass = [trainClass; trainCentroidClass];
    testMat = [testMat; testData{centroidCounter}];
    testCentroidClass = iClass * ones( size( testData{centroidCounter}, 1), 1 );
    testClass = [testClass; testCentroidClass];
    centroidCounter = centroidCounter + 1;
  end
end
trainRandIdx = randperm(size(trainMat,1));
trainMat = trainMat(trainRandIdx,:);
trainClass = trainClass(trainRandIdx);

return %END local_generateTrainTestData()