function [optimumNumCentroids, minErr] = optimizeNumberOfCentroidsMixtureSda( ...
  nCentroidsVec, simMat, classLabels, maxIter, nFolds)
%optimizeNumberOfCentroidsMixtureSda - cross-validate number of centroids.
% Cross validate the number of centroids to be used in a
% mixture SDA classifier for a two-class problem.
% Uses the cross-validation strategy in HTF, pg 216.
%
% USAGE
% [optimumNumCentroids, minErr] = optimizeNumberOfCentroidsJointSda( ...
%     maxNumCentroids, simMat, classLabels);
% nCentroidsVec: vector of number of centroids per class to test.
% simMat: square matrix of pairwise similarities.
% classLabels: vector of numeric class labels correpsonding to each row in
%   simMat. Only two classes are supported by this function, but their
%   labels can be any numeric value.
% nFolds: number of folds in cross validation. i.e. nFolds=5 means 5-fold
%   cross-validation, nFolds = size(simMat,1) means leave-one-out
%   cross-validation.
% optimumNumCentroids: 2-by-1 vector containing the optimum (minimum error)
%   number of centroids for each class
% minErr: classification error corresponding to the optimum number of
%   centroids.
%
% See also computeMultiCentroidModels(), classifyMultiNearestCentroid().

% Written by Luca Cazzanti, engineer.
% Applied Physics Laboratory, University of Washington, Feb 2007.
%$Id:$
%$DateTime:$

[nData, notUsed] = size(simMat); %#ok

uniqueClassLabels = unique(classLabels);
nClasses = length(uniqueClassLabels);
if nClasses ~= 2
  error('Only 2-class problems currently supported');
end

errMat = 10e10 * ones(length(nCentroidsVec));
for iCentroid = 1:length(nCentroidsVec)
  for jCentroid = 1:length(nCentroidsVec)
    
  classEst = 10e10 * ones(nData,1);
  nCentroidsPerClass = [nCentroidsVec(iCentroid) nCentroidsVec(jCentroid)];

  % New way: n-fold cross-validation, can also do leave-one-out
  classEst = local_NfoldCrossValidation(simMat, classLabels, ...
    nCentroidsPerClass, maxIter, nFolds);
  
  errMat(iCentroid, jCentroid) = sum( classEst(:) ~= classLabels(:) );
  end %for jCentroid
end %for iCentroid

[minErr1, bestNumIdx1] = min(errMat);
[minErr, bestNumIdx2] = min(minErr1);
bestNumCentroids1 = nCentroidsVec(bestNumIdx1(bestNumIdx2));
bestNumCentroids2 = nCentroidsVec(bestNumIdx2); 
optimumNumCentroids = [bestNumCentroids1 bestNumCentroids2];

return

%% LOCAL FUNCTIONS
function classEst = local_NfoldCrossValidation(simMat, classLabels, ...
  nCentroidsPerClass, maxIter, nFolds)

[nData, notUsed] = size(simMat); %#ok
nTest = floor(nData/nFolds);
classEst = [];
initIdx = 1;
for iFold = 1:nFolds
  testIdx = initIdx:(initIdx+nTest-1);
  testMatSim = simMat(testIdx,:);
  trainSimMat = simMat;
  trainSimMat(testIdx, :) = [];
  trainSimMat(:, testIdx) = [];
  trainClassLabels = classLabels;
  trainClassLabels(testIdx) = [];
  
  modelParams = computeMixtureSdaModels(trainSimMat, ...
    trainClassLabels, nCentroidsPerClass, 'maxSumSim', maxIter);
  classEstFold = classifyMixtureSda(testMatSim, modelParams);
  classEst = [classEst; classEstFold(:)];
  
  initIdx = initIdx + nTest;
end %for iFold