function [bestSize, minErr] = crossValidateNeighborhoodKnn( ....
 neighborhoodSizeVec, simMat, crossValIdxMat, classLabels, testIdx, simType)
%crossValidateNeighborhoodKnn
% REDO HELP
% Cross-validate the neighborhood size for the local SDA classifier, using
% the N-fold cross validation strategy. This function is designed to be
% used in conjucntion with readPartitionFile() and with the partition files
% from the Similarity Learning Repository.
%
% USAGE
% [bestSize, minErr] = crossvalidateNeighborhoodKnn( ...
%     neighborhoodSizeVec, simMat, crossValIdxMat, classLabels, ...
%     testIdx, simType)
% neighbrohoodSizeVec: vector of neighborhood sizes to cross-validate.
% simMat: N-by-N matrix of pairwise similarities.
% crossValIdxMat: N-by-K matrix of indices into the rows of simMat,
%    denoting the samples to be used for cross-validation. It is a cell
%    from crossValIdxCell as returned by readPartitionFile().
% classLabels: N-by-1 vector of class labels for each row of simMat.
% testIdx: a vector if indices into the columns of simMat denoting which
%     columns should be reserved for the testing data set.
% simType: string, either 'similarity' or 'dissimilarity'.
% bestSize: optimized neighborhood size.
% minErr: cross-validation error corresponding to best neighborhood size.

% Written by Luca Cazzanti, engineer.
% Applied Physics Laboratory, University of Washington, Feb 2009.
%$Id:$
%$DateTime:$
%$Author:$

neighborhoodSizeVec = sort(neighborhoodSizeVec);
nNeighborhoodSizes = length(neighborhoodSizeVec);

[nData, notUsed] = size(simMat); %#ok
[nFolds, nCrossVals1] = size(crossValIdxMat);

uniqueClassLabels = unique(classLabels);
nClasses = length(uniqueClassLabels);

neighborIdx = findNearNeighbors(simMat, size(simMat,2), simType);
neighborIdx = adjustSelfSimilarityRank(neighborIdx);
neighborIdx(:,1) = [];

errVec = 10e10 * ones(nNeighborhoodSizes,1); 
for iSize = 1:nNeighborhoodSizes

  thisSize = neighborhoodSizeVec(iSize);
  classEst = [];
  actualClass = [];
  for iFold = 1:nFolds
    crossValIdx = crossValIdxMat(iFold,:);
    crossValIdx(crossValIdx == 0) = []; % take out any remaining filler values
    nCrossVals = length(crossValIdx);
    
    thisFoldClassEst = -1e6 * ones(nCrossVals,1);
    thisFoldActualClass = 1e-6 * ones(nCrossVals,1);
    
    % iterate through all the "test" samples in this cross-validation fold
    for iCrossVal = 1:nCrossVals 
      
      % get the ith "test" sample's index
      thisCrossValSample = crossValIdx(iCrossVal);
      thisFoldActualClass(iCrossVal) = classLabels(thisCrossValSample);
      
      % pull out its neighbors
      neighborsToThisCrossValSample = neighborIdx(thisCrossValSample,:);
      
      % DEBUG CODE, DELETE FOR RUNNING
      %save neighborsToThisCrossValSample
      %save testIdx
      %save crossValIdx
      
      % Remove other cross validation "test" points as candidate neighbors
      % by looking through each of them and removing them
      for iCount = 1:nCrossVals
        idx = find( neighborsToThisCrossValSample == crossValIdx(iCount) );
        neighborsToThisCrossValSample(idx) = [];
      end % for iCount
      
      % Begin Maya's code fix
      % Remove the true test samples as candidate neighbors
      % by looking through each of them and removing them
      
      for iTest = 1:length(testIdx)
        idx = find( neighborsToThisCrossValSample == testIdx(iTest) );
        neighborsToThisCrossValSample(idx) = [];
      end % for iTest
      
      % end Maya's code fix
      
      neighborsToPick = neighborsToThisCrossValSample(1:thisSize);
      neighborClassLabels = classLabels(neighborsToPick);
      
      % Here's the classification rule - pick the most frequently
      % occurring class (majority vote)
      thisFoldClassEst(iCrossVal) = classifyKnn(neighborClassLabels, ...
        uniqueClassLabels);
            
    end %for iCrossVal
  
    classEst = [classEst; thisFoldClassEst(:)];
    actualClass = [actualClass; thisFoldActualClass(:)];
  end % for iFold
  
  errVec(iSize) = sum( classEst ~= actualClass );   
end %for iSize


[minErr, minErrIdx] = min(errVec);
bestSize = neighborhoodSizeVec(minErrIdx);

return