function [testIdxMat, crossValIdxCell, nTests, nFolds] = ... 
  readPartitionFile(fileName)
%readPartitionFile - reads in an experiment partition file.
% Read in a partition file and returns the indexes of the test and
% cross-validation samples for each test split. The partition must be in
% the Similarity Learning Repository format.
%
% USAGE
%[testIdxMat, crossvalIdxCell, nTests, nFolds] = ...
%    readPartitionFile(fileName);
% fileName - the name of the partition file
% testIdxMat - nTests-by-T matrix, each row contains the indices into the
%    columns of the similarity matrix denoting the samples reserved for
%    testing; nTests is the total number of test splits, T is the number of
%    test points in each split.
% crossValIdxCell - nTests-by-1 cell array. The i-th cell is a nFold-by-K
%    matrix whose rows contain the indices of the K elements to be used for
%    cross-validation for the i-th set of test samples.
% nTests - number of tests.
% nFolds - number of folds to use for each test for n-fold
% cross-validation.
  
% Written by Luca Cazzanti, February 19, 2008.
% Copyright Applied Physics Laboratory - University of Washington.

% Read in partition file, put -1 to fill out rows of different lenghts;
% will be taken out later.
partitionInfo = textread(fileName, '', 'emptyvalue', -1);
nTests = partitionInfo(1,1);
nFolds = partitionInfo(1,2);

%% Matrix of indexes of the test samples
% Each row contains the indexes of the test samples for a test split:
testIdxMat = partitionInfo(2:(nFolds+1):end,:);

% Quick check: there should be nTests rows in the matrix
if size(testIdxMat,1) ~= nTests
  error('Size of "testIdxMat" does not match number of test splits');
end

% Remove any -1 columns, which are just placeholders to accommodate rows of
% dirrerent lenghts in the *.idx file
testIdxMat = local_removeEmptyValue(testIdxMat, -1);

% Matlab starts indexing at 1, but config files were designed to work with 
% C++, which starts at 0, so take care of it here:
testIdxMat = testIdxMat + 1;

% Cell array of indexes of the cross-validation samples:
% Each element of the cell is a matrix of cross-validation indexes for a
% particular test split; each row of each matrix contains cross-val indexes
% for a specific cross-val fold
crossValIdxCell = cell(nTests,1);
startIdx = 3;
endIdx = startIdx + nFolds - 1;
for iTest = 1:nTests
  crossValRows = startIdx:endIdx;
  crossValIdxMat = partitionInfo(crossValRows,:);
  crossValIdxMat = local_removeEmptyValue(crossValIdxMat, -1);
  crossValIdxMat = crossValIdxMat + 1;
  
  crossValIdxCell{iTest} = crossValIdxMat;
  clear crossValIdxMat;
  
  startIdx = endIdx + 2;
  endIdx = startIdx + nFolds - 1;
end

return % end of main function

%% LOCAL FUNCTIONS
%local_removeEmptyValue
function mat = local_removeEmptyValue(mat, emptyValue)
finished = ~all(mat(:,end) == emptyValue);
while (finished == false)
  mat(:,end) = [];
  finished = ~all(mat(:,end) == emptyValue);
end % while
return 
% END local_removeEmptyValue