% run_nfold_crossval
% Test classification algorithms on the data from the Similarity Learning
% Repository. This script automates testing of several SDA-type algorithms
% on the repository data.

%% Prelims

% List of data sets to run: the data for each set are assumed in the
% directory indicated below, in the similarity learning repository format
% See the Similarty Learning Repository website for information on this
% format.

%dataDirList = {'patrol', 'Voting', 'Mirex07'};
%dataDirList = {'AuralSonar', 'Patrol', 'Protein', 'Voting'};
%dataDirList = {'InternetAds-linear'};
%dataDirList = {'Protein', 'Voting'};
dataDirList = {'AmazonBinary'};

% Which classifiers to run?
doLocalSda = true;
doAverageLocalSda = false;

% Append a string to the result file name to avoid overwriting results
timeStamp = datestr(now, 30); % for uniquely identifying runs

%% Cycle through the directories and run algorithms on the data
for iDir = 1:length(dataDirList);
  
  dataDir = dataDirList{iDir};
  disp(dataDir)

  % Read in similarity data and labels
  filePrefix = fullfile(dataDir,dataDir);
  data = readSimData(filePrefix);

  % The sda-type algorithms have been tested with symmetric symilarities:
  % symmetrize the sim matrix if it is not natively symmetric.
  tmp = data.S.';
  if any(data.S(:) - tmp(:))
    data.S = ( data.S + tmp) / 2; % Force symmetric similarity
  end

  simMat = full(data.S);
  classLabels = data.lbl;
  clear data

  % Correction to the patrol data: make self-similarities be maximum 
  if strcmpi(dataDir, 'Patrol')
    disp('Applying correction to Patrol data self-similarity');
    for iData = 1:size(simMat, 1)
      simMat(iData, iData) = max(simMat(:));
    end 
  end

  % Read in the indices for the train/test partitions
  partitionFile = fullfile(dataDir, [dataDir '-Testpct=20-nTests=20-nFold=10.idx']);
  [testIdxMat, crossValIdxCell, nTests, nFolds] = readPartitionFile(partitionFile);

  % File where to store results
  [notUsed, resultFileTmp, fileExt] = fileparts(partitionFile);
  resultFile = ['result_' timeStamp '_' resultFileTmp '.mat'];
  resultFile = fullfile(dataDir, resultFile);

  % Do we assume we know the similarity domain or not?
  % Pick one and stick with it.
  %simDomain = unique(simMat(:));

  simDomain = determineSimDomain(dataDir, simMat);

  % Parameters for each data set
  nSamples = length(classLabels);
  nTestSamplesAll = size(testIdxMat,2);
  nCrossValSamplesAll = size(crossValIdxCell{1},2);
  nTrainSamplesAll = nSamples - nTestSamplesAll - nCrossValSamplesAll;

  % Define the neighborhood sizes to test for local classifiers
  maxPwr2 = floor(log2(nTrainSamplesAll));
  neighborhoodSize= 2.^(1:maxPwr2);
  %neighborhoodSize = unique([1:16 neighborhoodSize]);
  % Limit size to 128
  maxSize = min(nTrainSamplesAll, 128);
  neighborhoodSize(neighborhoodSize > maxSize) = [];
 
   
  % FOR DEBUGGING OLY - COMMENT OUT FOR REAL RUN
  %nTests = 1;
  %END DEBUGGING

  %% Local SDA
  if doLocalSda == true
    
    disp(' ... local SDA ...');
    
    bestSizeLocalSda = -1e6 * ones(nTests,1);
    minErr = -1e6 * ones(nTests,1);
    errCountLocalSda = -1e6 * ones(nTests,1);
    for iTest = 1:nTests
      
      disp(['Test split ' num2str(iTest) ' ...']);

      % Cross-validate number of neighbors
      crossValIdxMat = crossValIdxCell{iTest};

      [bestSizeLocalSda(iTest), minErr(iTest)] = ...
        crossValidateNeighborhoodLocalSda( neighborhoodSize, simMat, ...
        crossValIdxMat, classLabels, simDomain, testIdxMat(iTest,:));

      % Test using cross-validated number of neighbors

      testIdx = testIdxMat(iTest,:);

      [errCountLocalSda(iTest), thisSplitClassEstimate] = ...
        evaluateTestsLocalSda(bestSizeLocalSda(iTest), simMat, testIdx, ...
        classLabels, simDomain);

      nTestSamples(iTest) = length(testIdx);

      

    end % for iTest

    avgBestSizeLocalSda = mean(bestSizeLocalSda) %#ok
    stdBestSizeLocalSda = std(bestSizeLocalSda,1) %#ok
    avgErrCountLocalSda = mean(errCountLocalSda) %#ok
    errPctLocalSda = errCountLocalSda(:) ./ nTestSamples(:); 
    avgErrPctLocalSda = mean(errPctLocalSda) %#ok
    stdErrCountLocalSda = std(errCountLocalSda,1) %#ok

    stdErrPctLocalSda = std(errPctLocalSda,1) %#ok

    

    if exist(resultFile, 'file')

      save(resultFile, '-append', 'bestSizeLocalSda', 'errCountLocalSda', ...
        'avgBestSizeLocalSda', 'stdBestSizeLocalSda', 'nTestSamples', ...
        'avgErrCountLocalSda', 'avgErrPctLocalSda', 'minErr', ...
        'stdErrCountLocalSda', 'stdErrPctLocalSda', 'neighborhoodSize');

    else

      save(resultFile, 'bestSizeLocalSda', 'errCountLocalSda', ...
        'avgBestSizeLocalSda', 'stdBestSizeLocalSda', 'nTestSamples', ...
        'avgErrCountLocalSda', 'avgErrPctLocalSda', 'minErr', ...
        'stdErrCountLocalSda', 'stdErrPctLocalSda', 'neighborhoodSize');
    end

  end % if doLocalSda



%% regularized SDA: average local SDA

if doAverageLocalSda == true

  disp(' ... average local SDA ...');
  neighborType = 'kTotal'

  if strcmp(neighborType, 'kPerClass')
    % Adjust neighborhood sizes to be per class
    maxNeighborhoodSize = determineMaxNeighborhoodSizeByClass(classLabels, ...
      crossValIdxCell, testIdxMat);
    thisNeighborhoodSize = [neighborhoodSize(:); maxNeighborhoodSize];
    thisNeighborhoodSize(thisNeighborhoodSize > maxNeighborhoodSize) = [];
    thisNeighborhoodSize = unique(thisNeighborhoodSize)
  else
    thisNeighborhoodSize = neighborhoodSize
  end
  

  %% FOR DEBUGGING
  %thisNeighborhoodSize = [1:2];
  %thisNeighborhoodSize = 1:2;
  %thisNeighborhoodSize = 1:5;
  %% END DEBUGGING

  bestSizeAverageLocalSda = cell(nTests,1);
  bestRegParamAverageLocalSda = cell(nTests,1);
  minErr = cell(nTests,1);
  errCountAverageLocalSda = -1e6 * ones(nTests,1);

  regParamVec = [1e-6 1e-3 0.01 0.1 0.5 0.9]
  %regParamVec = [1e-6 0.1];
  %regParamVec = 1e-6;
  %regParamVec = 0

  for iTest = 1:nTests

    disp(['Test split ' num2str(iTest) ' ...']);

    % Cross-validate number of neighbors

     crossValIdxMat = crossValIdxCell{iTest};
     [bestSizeAverageLocalSda{iTest}, bestRegParamAverageLocalSda{iTest},  minErr{iTest}] = ...
       crossValidateParamsAverageLocalSda( thisNeighborhoodSize, ...
       neighborType, regParamVec, ...
       simMat, crossValIdxMat, classLabels, simDomain, testIdxMat(iTest,:));
     
    % Test using cross-validated parameters
    % Compute average models on entire training set using the best
    % neighborhood size and the best regularization constant

    testIdx = testIdxMat(iTest,:);
    trainSimMat = simMat;
    trainSimMat(:, testIdx) = [];
    trainSimMat(testIdx,:) = [];
    trainClassLabels = classLabels;
    trainClassLabels(testIdx) = [];
    
    % avgMarginal
    averageLocalSdaModelParamsAvgMarginal{iTest} = computeAverageLocalSdaModels(...
      trainSimMat, trainClassLabels, 'maxSumSim', ...
      bestSizeAverageLocalSda{iTest}.avgMarginal, simDomain);
    regularizingParamsAvgMarginal{iTest} = createRegularizingParamStruct( ...
      averageLocalSdaModelParamsAvgMarginal{iTest},  bestSizeAverageLocalSda{iTest}.avgMarginal, ...
      bestRegParamAverageLocalSda{iTest}.avgMarginal);
    [errCount, notUsed, posteriors, nonRegModelParamsAvgMarginal{iTest}] = ...
      evaluateTestsAverageLocalSda(regularizingParamsAvgMarginal{iTest}, simMat, testIdx, ...
      classLabels);
    errCountAvgMarginal(iTest) = errCount.avgMarginal;
    for iPost = 1:length(posteriors)
      tmpPostAvgMarginal{iPost} = posteriors{iPost}.avgMarginal;
    end
    posteriorsAvgMarginal{iTest} = tmpPostAvgMarginal;
    
    if(0)
    % AvgMarginalUniformPriors - DEPRECATED
    averageLocalSdaModelParams = computeAverageLocalSdaModels(...
      trainSimMat, trainClassLabels, 'maxSumSim', ...
      bestSizeAverageLocalSda{iTest}.avgMarginalUniformPriors, simDomain);
    regularizingParams = createRegularizingParamStruct( ...
      averageLocalSdaModelParams,  bestSizeAverageLocalSda{iTest}.avgMarginalUniformPriors, ...
      bestRegParamAverageLocalSda{iTest}.avgMarginalUniformPriors);
    [errCount, thisSplitClassEstimate2] = ...
      evaluateTestsAverageLocalSda(regularizingParams, simMat, testIdx, ...
      classLabels);
    errCountAvgMarginalUniformPriors(iTest) = errCount.avgMarginalUniformPriors;
    end %if(0)
    
    % AvgJoint
    averageLocalSdaModelParamsAvgJoint{iTest} = computeAverageLocalSdaModels(...
      trainSimMat, trainClassLabels, 'maxSumSim', ...
      bestSizeAverageLocalSda{iTest}.avgJoint, simDomain);
    regularizingParamsAvgJoint{iTest} = createRegularizingParamStruct( ...
      averageLocalSdaModelParamsAvgJoint{iTest},  bestSizeAverageLocalSda{iTest}.avgJoint, ...
      bestRegParamAverageLocalSda{iTest}.avgJoint);
    [errCount, notUsed, posteriors, nonRegModelParamsAvgJoint{iTest}] = ...
      evaluateTestsAverageLocalSda(regularizingParamsAvgJoint{iTest}, simMat, testIdx, ...
      classLabels);
    errCountAvgJoint(iTest) = errCount.avgJoint;
    for iPost = 1:length(posteriors)
      tmpPostAvgJoint{iPost} = posteriors{iPost}.avgJoint;
    end
    posteriorsAvgJoint{iTest} = tmpPostAvgJoint;
    
    if(0)
    % AvgJointUniformPriors - DEPRECATED
    averageLocalSdaModelParams = computeAverageLocalSdaModels(...
      trainSimMat, trainClassLabels, 'maxSumSim', ...
      bestSizeAverageLocalSda{iTest}.avgJointUniformPriors, simDomain);
    regularizingParams = createRegularizingParamStruct( ...
      averageLocalSdaModelParams,  bestSizeAverageLocalSda{iTest}.avgJointUniformPriors, ...
      bestRegParamAverageLocalSda{iTest}.avgJointUniformPriors);
    [errCount, thisSplitClassEstimate2] = ...
      evaluateTestsAverageLocalSda(regularizingParams, simMat, testIdx, ...
      classLabels);
    errCountAvgJointUniformPriors(iTest) = errCount.avgJointUniformPriors;
    
    % avgSimToCentroids - SHOWN TO BE A BAD IDEA!
    averageLocalSdaModelParams = computeAverageLocalSdaModels(...
      trainSimMat, trainClassLabels, 'maxSumSim', ...
      bestSizeAverageLocalSda{iTest}.avgSimToCentroids, simDomain);
    regularizingParams = createRegularizingParamStruct( ...
      averageLocalSdaModelParams,  bestSizeAverageLocalSda{iTest}.avgSimToCentroids, ...
      bestRegParamAverageLocalSda{iTest}.avgSimToCentroids);
    [errCount, thisSplitClassEstimate2] = ...
      evaluateTestsAverageLocalSda(regularizingParams, simMat, testIdx, ...
      classLabels);
    errCountAvgSimToCentroids(iTest) = errCount.avgSimToCentroids;
    
    % AvgAcrossClasses - SO/SO PERFORMANCE
    averageLocalSdaModelParams = computeAverageLocalSdaModels(...
      trainSimMat, trainClassLabels, 'maxSumSim', ...
      bestSizeAverageLocalSda{iTest}.avgAcrossClasses, simDomain);
    regularizingParams = createRegularizingParamStruct( ...
      averageLocalSdaModelParams,  bestSizeAverageLocalSda{iTest}.avgAcrossClasses, ...
      bestRegParamAverageLocalSda{iTest}.avgAcrossClasses);
    [errCount, thisSplitClassEstimate2] = ...
      evaluateTestsAverageLocalSda(regularizingParams, simMat, testIdx, ...
      classLabels);
    errCountAvgAcrossClasses(iTest) = errCount.avgAcrossClasses;

    end %if(0)
    nTestSamples(iTest) = length(testIdx);

    disp(['Errors in this test split (out of ' num2str(nTestSamples(iTest)) ')']);
    disp(['    avgMarginal: ' num2str(errCountAvgMarginal(iTest))]);
    %disp(['    avgMarginalUniformPriors: ' num2str(errCountAvgMarginalUniformPriors(iTest))]);
    disp(['    avgJoint: ' num2str(errCountAvgJoint(iTest))]);
    %disp(['    avgJointUniformPriors: ' num2str(errCountAvgJointUniformPriors(iTest))]);
    %disp(['    avgSimToCentroids: ' num2str(errCountAvgSimToCentroids(iTest))]);
    %disp(['    avgAcrossClasses: ' num2str(errCountAvgAcrossClasses(iTest))]);

    save(resultFile)
    
  end % for iTest

  errPctAvgMarginal = mean(errCountAvgMarginal(:) ./ nTestSamples(:))
  errPctAvgJoint = mean(errCountAvgJoint(:) ./ nTestSamples(:))
  %errPctAvgAcrossClasses = mean(errCountAvgAcrossClasses(:) ./ nTestSamples(:))
  
  %errPctAvgMarginalUniformPriors = mean(errCountAvgMarginalUniformPriors(:) ./ nTestSamples(:))
  %errPctAvgJointUniformPriors = mean(errCountAvgJointUniformPriors(:) ./ nTestSamples(:))
  %errPctAvgSimToCentroids = mean(errCountAvgSimToCentroids(:) ./ nTestSamples(:))
  
  
  save(resultFile)
  


end % if doAverageLocalSda

end % for iDir
