function [classEst, logPosteriors, nonRegModelParams] = ...
  classifyRegularizedLambdaLocalSda(testPointSim, neighborSimMat, ...
  neighborClassLabels, regularizingParams)
%classifyRegularizedLambdaLocalSda - lambda-regularized local SDA classifier. 
% Classifies a test sample using the lambda-regularized local SDA classifier.
% USAGE
% [classEst, logPosteriors, nonRegModelParams] = ...
%    classifyRegularizedLambdaLocalSda(simVec, simMat, classLabels,...
%    regularizingParams)
% simVec: 1-by-M vector of test point similarities to each of the set points.
% simMat: M-by-M matrix of pairwise similarities of all set points
% classLabels: M-by-1 vector of class labels of the points in the set.
% regularizingParams: structure containing several parameters needed for
%    regularization and model estimation. Use
%    createREgularizedLambdaParamStruct() to create this structure/
% classEst - class estimate for the test point corresponding to simVec.
% logPosteriors - log of the posterior probabilites for the classes.
% modelParams - structure with the computed maximum entropy model
%     parameters used for classification. See computeMaxEntModels for
%     details.
%
% The function regularizes the lambda parameter of the local SDA model in
% two ways. If regularizingParams.lambdaType is 'average', it uses the
% average of the local SDA lambdas as the regularizing values. In this case
% the fields regularizingParams.lambda and regularizingParams.classLabels
% are ignored. If regularizingParams.lambdaType is 'global', it regularizes
% using the parameters regularizingParams.lambda, which are assumes to have
% been computed by computeSdaModels().
%
% See also createRegularizedLambdaParamStruct.

% Written by Luca Cazzanti, engineer.
% Applied Physics Laboratory, University of Washington, March 2009.
%$Id:$
%$DateTime:$
%$Author:$

%% Assign some parameters
lambdaType = regularizingParams.lambdaType;
regParam = regularizingParams.regParam;
regLambda = regularizingParams.regLambda;
regClassLabels = regularizingParams.classLabels;
simDomain = regularizingParams.simDomain;
centroidType = regularizingParams.centroidType;

%% Compute parameters for non-regularized local SDA
nonRegModelParams = computeMaxEntModels(neighborSimMat, neighborClassLabels, ...
  centroidType, simDomain);
nLocalClasses = length(nonRegModelParams.classLabels);

%% Which regularizing lambda are we using?
switch lower(lambdaType)
  
  % use the average of the local lambdas
  case 'average'

    tempLambda = nonRegModelParams.lambda(:);
    tempLambda(isinf(tempLambda)) = [];
    if ~isempty(tempLambda)
      newRegLambda = mean(tempLambda);
    else
      newRegLambda = 0;
    end
    
  % use the lambda provided as input
  case 'global' 
  
  % Must line up the input lambda with the local lambda matrix because the
  % local model might have less classes than the provided model.
  idxVec = [];
  for iClass = 1:nLocalClasses
    idx = find(regClassLabels == nonRegModelParams.classLabels(iClass));
    idxVec = [idxVec idx];
  end
  newRegLambda = regLambda(idxVec,:);
  newRegLambda = newRegLambda(:,idxVec);
  
  otherwise 
    error(['Unsupported lamda-regularization type: ' lambdaType]);
    
end %switch

%% Regularize the exponents of the local SDA models
modelParams = nonRegModelParams;
modelParams.lambda = ( 1 - regParam) * modelParams.lambda + ...
  regParam * newRegLambda;

% Adjust the scaling constant to ensure models are still pmfs
for gClass = 1:size(modelParams.gamma, 1)
  for hClass = 1:size(modelParams.gamma, 2)
    modelParams.gamma(gClass, hClass) = ...
      1 / sum(exp(simDomain * modelParams.lambda(gClass, hClass)));
  end 
end

%% Classify using the regularized model parameters
[classEst, logPosteriors] = classifySda(testPointSim, modelParams);

return
  