function [averageModelParams, localSdaModelParams] = ...
  computeAverageLocalSdaModels(simMat, classLabels, centroidType, ...
  neighborhoodSizeVec, varargin)
%computeAverageLocalSdaModels - compute average local SDA models.
% Computes the average of local SDA models for various neighborhood sizes.
% The average is over N samples, where N is the dimension of the input
% square similarity matrix. Each local SDA model is computed by leaving one
% sample out and computing its local SDA model parameters for a fixed
% neighborhood size from the remaining N-1 samples. Several average local models
% are computed, one for each desired neighborhood size. The neighborhood 
% type is specified as an input, it can be either k neighbors per class or
% k total neighbors. 
% USAGE
% [averageModelParams localSdaModelParams] = ...
%     computeAverageLocalSdaModels(simMat, classLabels, centroidType, ...
%     neighborhoodSizeVec, simDomain, neighborType)
%
% simMat - N-by-N matrix of pairwise similarities.
% classLabels - length-N vector of class labels corresponding to the
%     columns of simMat.
% centroidType - string denoting the type of centroid to use. Only
%     'maxSumSim' has been tested. 
% neighborhoodSizeVec - vector of neighborhood sizes to use for the average
%     models.
% simDomain - optional vector of possible similarity values.
% neighborType - optional, either 'KPerClass' or 'kTotal' (default).
% averageModelParams - structure containing the parameters for the average
%    local models. The fields are:
%    * avgMarginalConditionalLocalSdaPmfs: C-by-C-by-K cell array containing
%    the average local SDA pmfs, where C is the number of classes and K is
%    the neighborhood size. The {g,h,k} cell element is a probability mass
%    function vector for the similarity of sample x to the centroid of
%    class h, given that sample x is from class g. The pmfs are over the
%    the values in simDomain.
%    * neighborhoodSize: vector of neighborhood sizes used. This may be
%    different from neighborhoodSizeVec due to the size of the data.
%    * simDomain: vector of possible similarity values.
%    * centroidType: string denoting the type of centroid used, set to
%        'maxsumSim'.
%    * regType: string denoting the regularization used, set to
%        'averageLocalSda'.
%    * neighborType: string denoting the type of neighborhood used.
%    * classPriors: the class prior probabilities (Laplace correction applied)
%    * classCounts: number of data samples from each class. 
%    * classLabels: the possible class labeles.
%
% See also classifyRegularizedLocalSda.

% Written by Luca Cazzanti, engineer.
% Applied Physics Laboratory, University of Washington, Jan 2009.

% Determine the similarity domain
switch nargin
  case 4
    simDomain = unique(simMat(:));
  case 5 
    simDomain = unique(varargin{1});
    neighborType = 'kTotal';
  case 6
  simDomain = unique(varargin{1});
  neighborType = varargin{2};
  if isempty(neighborType)
    neighborType = 'kTotal';
  end
  otherwise
    error('Incorrect number of inputs');
end % swith nargin


% What kind of neighborhood should we use?
switch lower(neighborType)
  case 'kperclass'
    [neighborIdxCell, actualNeighborhoodSizeVec] = local_getKPerClassParams(simMat, ...
      classLabels, neighborhoodSizeVec);
  case 'ktotal'
    [neighborIdxCell, actualNeighborhoodSizeVec] = local_getKTotalParams(simMat, ...
      neighborhoodSizeVec);
  otherwise
    error(['Unsupported neighborhood type: ' neighborType]);
    return
end

nSizes = length(actualNeighborhoodSizeVec);

% Loop over neighborhood sizes, compute local SDA models for each size,
% each time treating a different sample as a test sample. Average the
% resulting size-dependent local SDA models over all the samples: these are
% the average local SDA models we are seeking.
nSamples = size(simMat, 1);
localSdaModelParams = cell(nSamples, nSizes);
simToCentroidHistPmfs = cell(nSamples, nSizes);

possibleClassLabels = unique(classLabels);
nClasses = length(possibleClassLabels);

localBayesPmfs = cell(nSamples, nSizes, nClasses);
tmpBayesPmfs = cell(nClasses,1);

for iSample = 1:nSamples
  thisSampleNeighborIdxAll = neighborIdxCell{iSample};

  for iSize = 1:nSizes
    % Define matrix of pairwise similarities between neighbors 
    nNeighbors = actualNeighborhoodSizeVec(iSize);
    neighborIdx = thisSampleNeighborIdxAll(:,1:nNeighbors);
    neighborIdx = neighborIdx(:);
    neighborSimMat = simMat(:, neighborIdx);
    neighborSimMat = neighborSimMat(neighborIdx,:);
    neighborClassLabels = classLabels(neighborIdx);
  
    % Compute class-conditional models
    modelParams = computeMaxEntModels(...
      neighborSimMat, neighborClassLabels, centroidType, simDomain);
    
    % Compute the unconditional pmfs of the similarity to the class 
    % centroids: they are just normalized frequency counts - DEPRECATED
    simHist = hist(neighborSimMat(:,modelParams.centroidIdx), simDomain);
    
    % Compute the naive Bayes pmfs - UNTESTED
    for gClass = 1:nClasses
      gIdx = find(neighborClassLabels == possibleClassLabels(gClass));
      if ~isempty(gIdx)
        if numel(gIdx) == 1
          for hClass = 1:length(modelParams.classLabels)
            tmp = hist(neighborSimMat(gIdx,modelParams.centroidIdx(hClass)), ...
              simDomain);
            tmpHist(:,hClass) = tmp(:);
          end
        else
          tmpHist = hist(neighborSimMat(gIdx,modelParams.centroidIdx), simDomain);
        end
        if size(tmpHist,1) == 1
          tmpHist = tmpHist.';
        end
        tmpBayesPmfs{gClass} = tmpHist;

      else
        tmpBayesPmfs{gClass} = zeros(length(simDomain), ...
          length(modelParams.classLabels));
      end
    end

        
    % Align the local model: this is needed if locally not all classes are
    % represented in the neighborhood
    if length(modelParams.classLabels) ~= nClasses
    
      alignedModelParams = alignLocalModel(modelParams, possibleClassLabels);
      localSdaModelParams{iSample, iSize} = alignedModelParams;
     
      % DEPRECATED
      alignedSimHist = zeros(1,nClasses);
      for gClass = 1:length(modelParams.classLabels);
        %idx = find(possibleClassLabels == modelParams.classLabels(gClass));
        idx = possibleClassLabels == modelParams.classLabels(gClass);
        alignedSimHist(idx) = simHist(gClass);
      end % for gClass
      simHistNorm = sum(alignedSimHist);
      simToCentroidHistPmfs{iSample, iSize} =  ...
        alignedSimHist ./ repmat(simHistNorm, size(alignedSimHist,1), 1);
      %END DEPRECATED

      for gClass= 1:nClasses
        alignedBayesPmfs = zeros(length(simDomain),nClasses);
        tmpBayesPmfsMat = tmpBayesPmfs{gClass};
        for hClass = 1:length(modelParams.classLabels);
          %idx = find(possibleClassLabels == modelParams.classLabels(hClass));
          idx = possibleClassLabels == modelParams.classLabels(hClass);
          alignedBayesPmfs(:,idx) = tmpBayesPmfsMat(:,hClass);
        end %for hClass
        normFactor = sum(alignedBayesPmfs);
        normFactor(normFactor == 0) = 1;
        alignedBayesPmfs = alignedBayesPmfs ./ repmat(normFactor, ...
          length(simDomain), 1);
        localBayesPmfs{iSample, iSize, gClass} = alignedBayesPmfs;
      end % for gClass
      
    else

      localSdaModelParams{iSample, iSize} = modelParams;
      
      simHistNorm = sum(simHist);
      simToCentroidHistPmfs{iSample, iSize} =  ...
        simHist ./ repmat(simHistNorm, size(simHist,1), 1);
    
      for gClass = 1:nClasses
        normFactor = sum(tmpBayesPmfs{gClass});
        normFactor(normFactor == 0) = 1;
        tmpBayesPmfs{gClass} = tmpBayesPmfs{gClass} ./ repmat(normFactor, ...
          length(simDomain), 1);
        localBayesPmfs{iSample, iSize, gClass} = tmpBayesPmfs{gClass};
      end% for gClass
    
    end % if-then-else
    
  end % for iSize
  
end % for iSample

%% Average the models over the samples

% Initialize the result arrays/cells
avgMarginalConditionalLocalSdaPmfs= cell(nClasses, nClasses, nSizes);
avgSimToCentroidHistPmfs = cell(1,nSizes);
avgAcrossClasses = cell(nClasses, nSizes);
avgMeanSim = cell(nSizes,1);
avgLocalBayesPmfs = cell(nClasses, nSizes);

for iSize = 1:nSizes
  for iClass = 1:nClasses 
    for jClass = 1:nClasses
      avgMarginalConditionalLocalSdaPmfs{iClass, jClass, iSize} = ...
        zeros(length(simDomain),1);
    end
    avgAcrossClasses{iClass, iSize} = zeros(length(simDomain),1);
    avgLocalBayesPmfs{iClass, iSize} = zeros(length(simDomain), nClasses);
  end
  avgSimToCentroidHistPmfs{iSize} = ...
    zeros(length(simDomain), nClasses);
  avgMeanSim{iSize} = zeros(nClasses, nClasses);
end

% Loop over the samples and get the average pmf for each neighborhood
% size;
avgClassPriors = zeros(nClasses,1);
for iSize = 1:nSizes

  for iSample = 1:nSamples
    modelParams = localSdaModelParams{iSample, iSize};
    
    % This is for checking degenerate cases where only 1 similarity value
    % appears in a local class. Used inside inner loop.
    infiniteLambdaIdx = modelParams.lambda == -Inf; %logical matrix
 
    % Iterate over the classes and compute the pmf vector
    
    for gClass = 1:nClasses
            
      for hClass = 1:nClasses
    
        % Case in which only one similarity value was observed
        if infiniteLambdaIdx(gClass, hClass) == true
          pmfVector = zeros(length(simDomain), 1);
          pmfVector(simDomain(:) == modelParams.meanSim(gClass, hClass)) = 1;
        else
          pmfVector = modelParams.gamma(gClass, hClass) * ...
            exp(modelParams.lambda(gClass,hClass) * simDomain);
        end
        
        % Accumulate the pmfs for each (g,h) class combination, scaled by
        % the number of samples. At the end of the loop, the result is the
        % average pmf for each (g,h)
        avgMarginalConditionalLocalSdaPmfs{gClass, hClass, iSize} = ...
          avgMarginalConditionalLocalSdaPmfs{gClass, hClass, iSize} + ...
          pmfVector/nSamples;
        
      end % for hClass
                  
      avgLocalBayesPmfs{gClass, iSize} = ...
        avgLocalBayesPmfs{gClass, iSize} + ...
        localBayesPmfs{iSample, iSize, gClass} / nSamples;
    end % for gClass
     
    avgClassPriors = avgClassPriors(:) + modelParams.classPriors(:) / nSamples;
    
    % REVIEW THIS
    avgMeanSim{iSize} = avgMeanSim{iSize} + modelParams.meanSim / nSamples;
     
  end % for iSample
 
  %Re-normalize the pmfs: this needs to be done if the local models were
  %re-aligned in the previous for hClass loop, because it is possible to
  %add pmfVector = all zeros, which makes sum(average pmf) < 1.
  for gClass=  1:nClasses
    
    for hClass = 1:nClasses
      
      % Check is pmf is degenerate to all zeros, This happens if no
      %(gClass, hClass) is observed locally. Then we set the probability to
      %the uniform over the size of the sim domain. Other possibilities are
      %setting it to 1 or 0. Thanks to Peter for catching this potential
      %problem. LGC 11/24/2009.
      tmpSum = sum(avgMarginalConditionalLocalSdaPmfs{gClass, hClass, iSize});
      if tmpSum < eps
        avgMarginalConditionalLocalSdaPmfs{gClass, hClass, iSize} = ...
          1/length(simDomain) * ...
          ones(size(avgMarginalConditionalLocalSdaPmfs{gClass, hClass, iSize}));
      else
      avgMarginalConditionalLocalSdaPmfs{gClass, hClass, iSize} = ...
        avgMarginalConditionalLocalSdaPmfs{gClass, hClass, iSize} / tmpSum;
      end % if-then-else
      
    end % for hClass
    
    tmpBayesMat = avgLocalBayesPmfs{gClass, iSize};
    tmpNorm = sum(tmpBayesMat);
    tmpNorm(tmpNorm == 0) = 1;
    tmpBayesMat = tmpBayesMat ./ repmat(tmpNorm, length(simDomain),1);
    avgLocalBayesPmfs{gClass, iSize} = tmpBayesMat;
  
  end % for gClass
  
  % Average over the classes - NOT THAT USEFUL FOR CLASSIFICATION
  for hClass = 1:nClasses
    for gClass = 1:nClasses
      avgAcrossClasses{hClass, iSize} = ...
        avgAcrossClasses{hClass, iSize} +  ...
        avgMarginalConditionalLocalSdaPmfs{gClass, hClass, iSize} * ...
        avgClassPriors(gClass);
    end % for gClass
  end % for hClass
    
end % for iSize

%% Compute the class priors
% Computed from the entire data set
classCounts = hist(classLabels, unique(classLabels));
classPriors = (classCounts + 1 )/( sum(classCounts) + nClasses); % note Laplace correction

%% Store results in a structure
averageModelParams.avgMarginalConditionalLocalSdaPmfs = ...
  avgMarginalConditionalLocalSdaPmfs;
averageModelParams.neighborhoodSizes = actualNeighborhoodSizeVec;
averageModelParams.simDomain = simDomain;
averageModelParams.centroidType = centroidType;
averageModelParams.regType = 'averageLocalSda';
averageModelParams.neighborType = lower(neighborType);
averageModelParams.classCounts = classCounts;
averageModelParams.classPriors = classPriors; % computed form entire training set
averageModelParams.avgClassPriors = avgClassPriors; %average of local priors
averageModelParams.classLabels = unique(classLabels);

% These fields are not that useful for regularization (either not that
% helpful, or they make classification worse) but are kept here 
% for backward compatibility.
averageModelParams.avgSimToCentroidHistPmfs = avgSimToCentroidHistPmfs;
averageModelParams.avgAcrossClasses = avgAcrossClasses;

%These are preliminary and not yet fully functional
averageModelParams.avgMeanSim = avgMeanSim; % check this is computed correctly
averageModelParams.avgLocalBayesPmfs = avgLocalBayesPmfs;

return %END computeAverageLocalSdaModels

%% LOCAL FUNCTIONS
% local_getKPerClassParams()
function [neighborIdx, actualNeighborhoodSizeVec] = ...
  local_getKPerClassParams(simMat, classLabels, neighborhoodSizeVec)

neighborIdx = findNearNeighborsByClass(simMat, size(simMat,2), ...
  classLabels, 'similarity');
neighborIdx = adjustSelfSimilarityRankByClass(neighborIdx, ...
  classLabels, 'remove');

% Determine max possible neighborhodd size and adjust vector of sizes to
% try accrodingly
maxNeighbor = size(neighborIdx{1},2); 
maxNeighbor = min([max(neighborhoodSizeVec) maxNeighbor]);
actualNeighborhoodSizeVec = [neighborhoodSizeVec(:); maxNeighbor];
actualNeighborhoodSizeVec(actualNeighborhoodSizeVec > maxNeighbor) = [];
actualNeighborhoodSizeVec = unique(actualNeighborhoodSizeVec(:));

return
% END local_getKPerClassParams()

% local_getKTotalParams()
function [neighborIdx, actualNeighborhoodSizeVec] = ...
  local_getKTotalParams(simMat, neighborhoodSizeVec)

neighborIdxMat = findNearNeighbors(simMat, size(simMat,2), 'similarity');
neighborIdxMat = adjustSelfSimilarityRank(neighborIdxMat);
neighborIdxMat(:,1) = []; % remove self-similarity 

% Determine max possible neighborhodd size and adjust vector of sizes to
% try accrodingly
maxNeighbor = size(neighborIdxMat,2); 
maxNeighbor = min([max(neighborhoodSizeVec) maxNeighbor]);
actualNeighborhoodSizeVec = [neighborhoodSizeVec(:); maxNeighbor];
actualNeighborhoodSizeVec(actualNeighborhoodSizeVec > maxNeighbor) = [];
actualNeighborhoodSizeVec = unique(actualNeighborhoodSizeVec(:));

% Make the matrix of neighbor indices into a cell array to be consistent
% with the k-per-class neighborhood funciton interface: this makes the main
% function work seamlessly for both types of neighborhoods
nSamples = size(neighborIdxMat,1);
neighborIdx = cell(nSamples,1);
for iSample = 1:nSamples
  neighborIdx{iSample} = neighborIdxMat(iSample,:);
end

return
%END local_getKTotalParams()


%% IDEA - POSSIBLE EXTENSION/IMPROVEMENT
% Fit an exponential to each average local pmfs for each neighborhood size
% under the constraint that the mean be equal to the expected value given
% by the local average pmf.
% Pros:
% * seamless extension to unobserved similarities
% * consistency with non-regularized SDA models
% * the equivalent mean constraint may inform how different the regularizing
%   and non-regularizing model estimates are
% * can compare to regularized mean strategy by comparing the means
% Cons:
% average local pmfs may not be close to exponential so fitting may
% be too biased to be useful

 