function [modelParams, modelParamsStorage, loglikeStorage] = ...
  computeMixtureSdaModels( simMat, classLabels, ...
  nCentroidsPerClass, centroidType, maxIter, varargin)
%computeMixtureSdaModels -  compute mixture SDA model parameters.
% Computes the maximum entropy model parameters (lambda and gamma),
% the mixture component weights, and the mixture component centroids for
% mixture SDA class models. Only two-class problems are supported.
% 
% USAGE
% modelParams = computeJointSdaModels( simMat, classLabels, ...
%     nCentroidsPerClass, centroidType, maxIter, simDomain);
% modelParams: structure containing the model parameters. It is
%     identical to the structure returned by computeMixtureMaxEntModels,
%     with the additional field centroidWeights, a 2-by-1 cell array of
%     weights associated with each class.
% maxIter: maximum number of iterations.
% For details on the other input and output arguments see
% computeMixtureMaxEntModels().
% See also classifyMixtureSda().

% Written by Luca Cazzanti, engineer
% Applied Physics Laboratory, University of Washington, Jan 2007.
%$Id:$
%$DateTime:$
%$Author:$

uniqueClassLabels = unique(classLabels);
nClasses = length(uniqueClassLabels);
if nClasses == 1 % If only one class, return the trivial class model
  % Initialize model parameters struct with empty fields
  modelParams = initSdaModels;
  
  modelParams.isTrivial = true;
  modelParams.classCounts = size(simMat,1);
  modelParams.classPriors = 1; 
  modelParams.classLabels = uniqueClassLabels;
  modelParams.centroidWeights = cell(nClasses,1);
  
  % All other fields are empty 
  return
end % if nClasses

% Optimize the centroids first 
initCentroidIdx = initMultiCentroids(nCentroidsPerClass, classLabels);
modelParams = computeMultiCentroidModels(simMat, classLabels, ...
  initCentroidIdx, centroidType);

% Initialize the EM responsibilities to be 1/(num components) in each
% class; this is the same as initializing the component weights to 
% 1/(num components).
modelParams.centroidWeights = cell(nClasses,1);

if isscalar(nCentroidsPerClass)
  nCentroidsPerClass = nCentroidsPerClass * ones(nClasses, 1);
elseif length(nCentroidsPerClass) ~= nClasses
  error('Inconsistent number of classes');
end
for iClass = 1:nClasses
  modelParams.centroidWeights{iClass} = 1/nCentroidsPerClass(iClass) * ...
    ones(nClasses, nCentroidsPerClass(iClass));
  modelParams.componentPriors{iClass} = 1/nCentroidsPerClass(iClass) * ...
    ones(nClasses, nCentroidsPerClass(iClass));
end

% Compute first iteration, based on uniform component weights
modelParams = computeMixtureMaxEntModels(simMat, classLabels, ...
    modelParams, varargin{:});

% Iterate model updates
loglikeStorage = zeros(maxIter, nClasses);
modelParamsStorage = cell(maxIter);

for iIter = 1:maxIter

  % Update the weights
  componentWeights = computeMixtureSdaWeights(modelParams, simMat, ...
    classLabels, nClasses);
  modelParams.centroidWeights = componentWeights;

  % Update SDA parameters
  modelParams = computeMixtureMaxEntModels(simMat, classLabels, ...
    modelParams, varargin{:});

  % BEGIN ADAPTIVE CENTROID SECTION - NOT READY YET! 
  % Centroids can be adapted in each iteration
  if(0) % set to 1 to adapt centroids
  newCentroidIdx = [];
  newCentroidClassLabels = [];
  for iClass = 1:nClasses
    thisClassIdx =  classLabels == modelParams.classLabels(iClass);
    thisClassSimMat = simMat(thisClassIdx, thisClassIdx);
    thisClassLambda = modelParams.lambda{iClass};
    thisClassGamma = modelParams.gamma{iClass};
    thisClassCentroidWeights = modelParams.centroidWeights{iClass};
    thisClassNumCentroids = size(thisClassCentroidWeights,2);

    selfClassLambda = thisClassLambda(iClass,:);
    selfClassGamma = thisClassGamma(iClass,:);
    selfClassCentroidWeights = thisClassCentroidWeights(iClass,:);

    thisClassNewCentroidIdx = zeros(1, thisClassNumCentroids);
    for iCentroid = 1:thisClassNumCentroids

      thisCentroidExponentMat = selfClassLambda(iCentroid) * thisClassSimMat;
      thisCentroidProbMat = selfClassGamma(iCentroid) * exp(thisCentroidExponentMat);
      thisCentroidResponsibility = selfClassCentroidWeights(iCentroid) * ...
        thisCentroidProbMat;
      tempExpectedValue = sum(thisClassSimMat .* thisCentroidResponsibility);
      [maxVal, thisClassNewCentroidIdx(iCentroid)] = max(tempExpectedValue); %#ok

    end %for iCentroid
    
    % Eliminate duplicates and sort
    % thisClassNewCentroidIdx = unique(thisClassNewCentroidIdx);
    thisClassNewCentroidIdx = sort(thisClassNewCentroidIdx);
    numNewCentroids = length(thisClassNewCentroidIdx);
    
    % Convert index into this class simMat --> index into overall simMat
    try
      tmpIdx = find(thisClassIdx);
      tmpIdx = tmpIdx(:).';
      thisClassNewCentroidIdx = tmpIdx(thisClassNewCentroidIdx);
      newCentroidIdx = [newCentroidIdx thisClassNewCentroidIdx];
      centroidClassLabels = modelParams.classLabels(iClass) * ones(1, numNewCentroids);
      newCentroidClassLabels = [newCentroidClassLabels centroidClassLabels];
    catch
      keyboard
    end
  end %for iClass
  
  modelParams.centroidIdx = newCentroidIdx;
  modelParams.centroidClassLabels = newCentroidClassLabels;
  
  end %if(0)
  %END ADAPTIVE CENTROIDS SECTION
  
  % Compute the log-likelihood 
  %disp(num2str(iIter))
  [notUsed, loglike] = classifyMixtureSda(simMat, modelParams);
  loglike1 = sum(loglike(classLabels == 1,1));
  loglike2 = sum(loglike(classLabels == 2,2));
  loglikeTotal = [loglike1 loglike2];
  
  modelParamsStorage{iIter} = modelParams;
  loglikeStorage(iIter,:) = loglikeTotal;
  
end % for iIter


return 
