function componentWeights = computeMixtureSdaWeights(modelParams, ...
  simMat, classLabels, nClasses)
%computeMixtureSdaWeights - compute component weights for mixture SDA.
% Computes the weights of the components (mixing probabilities) in mixture
% SDA class models. Uses the standard maximum likelihood/EM formula for
% computing the component weights of mixture models.
% USAGE
% componentWeights = computeMixtureSdaWeights(modelParams, simMat, ...
%    classLabels, nClasses)
% modelParams: structure containing the mixture SDA model parameters, as
%    returned by computeMixtureMaxEntModels.
% simMat: N-by-N pairwise training similarity matrix.
% classLabels: N-by-1 vector of class labels associated with each row of
%    simMat.

% Written by Luca Cazzanti, engineer
% Applied Physics Laboratory, University of Washington, March 2007.
%$Id:$
%$DateTime:$
%$Author:$

previousWeights = modelParams.centroidWeights;
componentPriors = modelParams.componentPriors;

% Iterate over the class-conditional models
componentWeights = cell(nClasses,1);
for iClassCentroid = 1:nClasses
  thisClassCentroidIdx = ( modelParams.centroidClassLabels == ...
    modelParams.classLabels(iClassCentroid) );
  thisClassCentroidIdx = modelParams.centroidIdx(thisClassCentroidIdx);
  thisClassNumCentroids = length(thisClassCentroidIdx);
  tempLambda = modelParams.lambda{iClassCentroid};
  tempGamma = modelParams.gamma{iClassCentroid};
  thisClassComponentWeights = previousWeights{iClassCentroid};
  thisClassComponentPriors = componentPriors{iClassCentroid};
  
  % Iterate over the different classes for the training data
  thisClassWeights = zeros(nClasses, thisClassNumCentroids);
  for iClassData = 1:nClasses
    thisClassLambda = tempLambda(iClassData,:);
    thisClassGamma = tempGamma(iClassData,:);
    thisClassIdx = classLabels == modelParams.classLabels(iClassData);
    simToCentroids = simMat(thisClassIdx, thisClassCentroidIdx);
    thisClassNumData = sum(thisClassIdx);
       
    % Iterate over data in this class
    responsibilities = zeros(thisClassNumData, thisClassNumCentroids);
    for iData = 1:thisClassNumData
      lambdaVal = thisClassLambda;
      gammaVal = thisClassGamma;
      thisSimVec = simToCentroids(iData,:);
         
      responsibilities(iData,:) = computeResponsibility(thisSimVec, ...
        lambdaVal, gammaVal, thisClassComponentWeights(iClassData,:), ...
        thisClassComponentPriors(iClassData,:));
      
    end %for iData
    
    thisClassWeights(iClassData,:) = mean(responsibilities);
    
    % DEBUGGING: take this out when finished debugging
    % Weights must add to 1
    %if sum(thisClassWeights(iClassData,:) ) ~= 1
    %  keyboard
    %end
        
  end %for iClassData
  
  componentWeights{iClassCentroid} = thisClassWeights;

end %for iClassCentroid

return